/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package com.ibm.etools.sfm.ui.customimportwizards.importWizards;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;

import com.ibm.etools.sfm.sfpi.SFChannelGenerationProperties;
import com.ibm.etools.sfm.sfpi.SFChannelInput;
import com.ibm.etools.sfm.sfpi.SFChannelOperation;
import com.ibm.etools.sfm.sfpi.SFChannelOutput;
import com.ibm.etools.sfm.sfpi.SFCommareaGenerationProperties;
import com.ibm.etools.sfm.sfpi.SFFault;
import com.ibm.etools.sfm.sfpi.SFGenerationProperties;
import com.ibm.etools.sfm.sfpi.SFInput;
import com.ibm.etools.sfm.sfpi.SFMQSeriesGenerationProperties;
import com.ibm.etools.sfm.sfpi.SFMessage;
import com.ibm.etools.sfm.sfpi.SFMessageFactory;
import com.ibm.etools.sfm.sfpi.SFMessageFile;
import com.ibm.etools.sfm.sfpi.SFOperationFactory;
import com.ibm.etools.sfm.sfpi.SFOperation;
import com.ibm.etools.sfm.sfpi.SFOperationsFile;
import com.ibm.etools.sfm.sfpi.SFOutput;

/**
 * This is an example custom import wizard that uses the SFPI.
 * To use this import wizard, first run this plugin on eclipse.
 * Then open the Enterprise Service Tools perspective, right click on the 
 * Nonterminal subproject of an SFM project, and choose Import->Import Message.
 * Then enter anything for the file name text box and press Finish.
 * This will create an operations file with operations and a message file with messages 
 * 		in the Nonterminal subproject.
 * @author Robert Hagan
 * @version 6.19.07
 */
public class ImportWizard extends Wizard implements IImportWizard {
	
	ImportWizardPage mainPage;

	public ImportWizard() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish() {
		IFile file = mainPage.createNewFile();
        if (file == null)
            return false;

        try {
        	
			IProject project = file.getProject();
			//create a message file
			SFMessageFile messageFile = SFMessageFactory.createMessageFile(
					project, getPostfixedFileName("messageFile", project, true));
			
			//create an input message
			SFMessage message1 = SFMessageFactory.createMessage(
					"inputMessage");
			
			try {
				//add some arbitrary elements to the message
				message1.addFloatingPointElement("float1", true, 4, 4, .0123);
				message1.addIntegerElement("int1", false, 4, 8, 5);
				message1.addStringElement("inputStr", 4, "yes");
				messageFile.addMessage(message1);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			//create an output message
			SFMessage message2 = SFMessageFactory.createMessage(
					"outputMessage");
			
			try {
				message2.addFloatingPointElement("output1", true, 4, 4, .0123);
				message2.addIntegerElement("outputInt1", false, 4, 8, 5);
				message2.addStringElement("outputStr", 4, "yes");
				messageFile.addMessage(message2);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			//create a fault message
			SFMessage message3 = SFMessageFactory.createMessage(
					"faultMessage");
			
			try {
				//add some arbitrary elements to the message
				message3.addFloatingPointElement("fault1", true, 4, 4, .0123);
				message3.addIntegerElement("faultInt1", false, 4, 8, 5);
				message3.addStringElement("faultStr", 4, "yes");
				messageFile.addMessage(message3);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			//create another fault message
			SFMessage message4 = SFMessageFactory.createMessage(
					"newfaultMessage");
			
			try {
				//add some arbitrary elements to the message
				message4.addFloatingPointElement("newfault1", true, 4, 4, .0123);
				message4.addIntegerElement("newfaultInt1", false, 4, 8, 5);
				message4.addStringElement("newfaultStr", 4, "yes");
				messageFile.addMessage(message4);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			
			//save the previous changes
			messageFile.save();
			
			ArrayList<SFMessage> list1 = new ArrayList<SFMessage>();
			list1.add(message1);
			list1.add(message3);
			ArrayList<SFMessage> list2 = new ArrayList<SFMessage>();
			list2.add(message2);
			list2.add(message3);			

			//create an operations file
			SFOperationsFile opFile = 
				SFOperationFactory.createOperationsFile(project, 
						getPostfixedFileName("opFile", project, false));
			
			try {
			//create channel input and output
			SFChannelInput chanInput = SFOperationFactory.createChannelInput(list1);
			SFChannelOutput chanOutput = SFOperationFactory.createChannelOutput(list2);
			
			//create a channel operation
			SFChannelOperation chanOp1 = SFOperationFactory.createChannelOperation("channelOp");
			chanOp1.setInput(chanInput);
			chanOp1.setOutput(chanOutput);
			
			//create generation properties for the channel operation
			SFGenerationProperties chanProps = SFOperationFactory.createGenerationProperties(chanOp1, 
					SFGenerationProperties.Runtime_CSFR32, SFGenerationProperties.InvokeType_Channel);
			
			SFChannelGenerationProperties chanProps2 = ((SFChannelGenerationProperties)chanProps);
			chanProps2.setChannelAdapterName("aChnAdapter");
			chanProps2.setChannelAdapterTranId("aChnAdid");
			chanProps2.setChannelLinkToProgram("aChnLink");
			chanProps2.setChannelLinkToTranId("aChnlLinkID");
			chanProps2.setChannelName("aChnName");
			chanProps2.setChannelSyncOnReturn(false);
			chanProps2.setChannelSysId("aChnSysID");
			chanProps2.setInputContainerProperties(message1, "inputContainer1");
			chanProps2.setOutputContainerProperties(message2, "outputContainer1", true);
			chanProps2.setInputContainerProperties(message3, "inputContainer2");
			chanProps2.setOutputContainerProperties(message3, "outputContainer2", false);
			chanOp1.setGenerationProperties(chanProps2);
			opFile.addOperation(chanOp1);
			
			// example of creating faults that could be set on an operation using the method setFaults
			SFFault fault1 = SFOperationFactory.createFault(message3);
			SFFault fault2 = SFOperationFactory.createFault(message4);
			ArrayList<SFFault> faultList = new ArrayList<SFFault>();
			faultList.add(fault1);
			faultList.add(fault2);

			}
			catch (Exception e) {
			e.printStackTrace();
			}

			try {
			SFInput input = SFOperationFactory.createInput(message1);
			SFOutput output = SFOperationFactory.createOutput(message2);

			//create a Link with COMMAREA operation
			SFOperation op1 = SFOperationFactory.createOperation("newCommOp");
			op1.setInput(input);
			op1.setOutput(output);
			SFGenerationProperties opProps = SFOperationFactory.createGenerationProperties(op1, 
					SFGenerationProperties.Runtime_CSFR32, SFGenerationProperties.InvokeType_COMMAREA);
			op1.setGenerationProperties(opProps);
			
			SFCommareaGenerationProperties opProps2 = ((SFCommareaGenerationProperties)opProps);
			opProps2.setCommareaAdapterName("aCommAdapter");
			opProps2.setCommareaAdapterTranId("aCommAdid");
			opProps2.setCommareaLinkToProgram("aCommLink");
			opProps2.setCommareaLinkToTranId("aCommlLinkID");
			opProps2.setCommareaMaxLength(174);
			opProps2.setCommareaSyncOnReturn(false);
			opProps2.setCommareaSysId("aCommSysID");
			op1.setGenerationProperties(opProps2);
			opFile.addOperation(op1);
			}
			catch (Exception e) {e.printStackTrace(); }
			
			try {
//			create an MQSeries operation
			SFOperation op2 = SFOperationFactory.createOperation("newMQOp");
			SFInput input2 = SFOperationFactory.createInput(message1);
			SFOutput output2 = SFOperationFactory.createOutput(message2);
			op2.setInput(input2);
			op2.setOutput(output2);
			SFGenerationProperties opMQProps = SFOperationFactory.createGenerationProperties(op2, 
					SFGenerationProperties.Runtime_CSFR32, SFGenerationProperties.InvokeType_MQSeries);
			
			SFMQSeriesGenerationProperties opMQProps2 = ((SFMQSeriesGenerationProperties)opMQProps);
			opMQProps2.setMQGetAdapterName("mqAdapter");
			opMQProps2.setMQGetTranId("mqAdid");
			opMQProps2.setMQMaxOutMsgLen(5);
			opMQProps2.setMQMessageType(SFMQSeriesGenerationProperties.MQMessageType_Request);
			opMQProps2.setMQPutAdapterName("putName");
			opMQProps2.setMQPutReplyQMgr("qmgr");
			opMQProps2.setMQPutReplyQueue("repQueue");
			opMQProps2.setMQPutRequestQueue("requQueue");
			opMQProps2.setMQPutTranId("putTranID");
			opMQProps2.setMQWaitInterval(5);
			op2.setGenerationProperties(opMQProps);
			opFile.addOperation(op2);
			
			}
			catch (Exception e) {e.printStackTrace();}

			//save the above changes to the file
			opFile.save();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
        return true;
	}
	 
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		setWindowTitle("File Import Wizard"); //NON-NLS-1
		setNeedsProgressMonitor(true);
		mainPage = new ImportWizardPage("Import Message",selection); //NON-NLS-1
	}
	
	/* (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizard#addPages()
     */
    public void addPages() {
        super.addPages(); 
        addPage(mainPage);        
    }
    
    /**
     * Returns a file name with the next number affixed to end.
     * (If there is a file1 and file2, this will return file3).
     * @param filename		The root name without a postfix
     * @param project		The project to check for the file in
     * @param isMessageFile		if true, checks for existing message files, 
     * 							if false, checks for existing operations files
     * @return the name to use
     */
    public String getPostfixedFileName(String filename, IProject project, boolean isMessageFile) {
    	int postfix = 1;
    	String extension = (isMessageFile) ? ".mxsd" : ".wsdl";
    	String filename2 = filename + String.valueOf(postfix) + extension;
    	IFolder folder = null;
    	//get the actual folder containing the files
    	if (isMessageFile) //message folder
    		folder = project.getFolder("Schema");
    	else	//operations folder
    		folder = project.getFolder("wsdl");

    	if (folder.exists()) {
    		IFile file = folder.getFile(filename2);
    		//get the right number for postfix
    		while (file.exists()) {
    			postfix++;
    			filename2 = filename + String.valueOf(postfix) + extension;
    			file = folder.getFile(filename2);
    		}
    	}
    	
    	return filename + String.valueOf(postfix);
    }
}
